@@ -16,9 +16,12 @@ MAX_S3_WORKERS = 20
 
 
 def get_args():
+    description = "S3 Parallel Action\n\n"
+    description += "Output:\n Key, Key Version, Key Size, Key Date, Is Latest/Current, Number of returned objects, Size of returned objects, Action Status"
     parser = argparse.ArgumentParser(
-        description="S3 Parallel Action",
-        epilog="Note: rm action using versions option remove specific object version and do NOT create delete marker!",
+        description=description,
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+        epilog="Warning Note: rm 'action' using 'versions' option remove specific object version/s and DO NOT CREATE DELETE MARKER!!!",
     )
 
     # common parser
@@ -125,13 +128,6 @@ def execute_s3_action(args, kwargs, client, data):
     else:
         prefix = ""
 
-    if args.skip_current_version and data["latest"]:
-        # skip current
-        return
-    if not args.versions and not data["latest"]:
-        # skip versions
-        return
-
     try:
         if args.dry or args.action == "ls":
             pass
@@ -147,7 +143,6 @@ def execute_s3_action(args, kwargs, client, data):
                 kwargs["CopySource"]["VersionId"] = version_id
             resp = client.copy_object(**kwargs)
 
-        # print(kwargs)
     except Exception as e:
         status = f"ERROR [{e}]"
     else:
@@ -163,7 +158,6 @@ def execute_s3_action(args, kwargs, client, data):
         "S": s_tot,
         "STATUS": status,
     }
-    # return f"KEY: {prefix}{key}; KV: {version_id} [{is_latest}]; KS: {key_size}; KD: {date}; N: {n_tot:n}; S: {s_tot}; STATUS: {status}"
 
 
 def get_kwargs_clients(args):
@@ -186,32 +180,25 @@ def get_kwargs_clients(args):
     return k_s3_ls, k_s3_act
 
 
-def get_kwargs_acts(args):
-    k_ls = {}
-    k_act = {}
-
-    k_ls["Bucket"] = args.bucket
-    k_act["Bucket"] = args.bucket
-
+def get_kwargs_ls(args):
+    k = {"Bucket": args.bucket}
     if args.prefix:
-        k_ls["Prefix"] = args.prefix
+        k["Prefix"] = args.prefix
     if args.start_after:
-        k_ls["KeyMarker"] = args.start_after
-    if args.start_after and args.version_id_marker:
-        k_ls["VersionIdMarker"] = args.version_id_marker
-    if args.key:
-        k_ls["Key"] = args.key
-        k_ls["ObjectAttributes"] = ["ObjectSize"]
-        if args.key_version:
-            k_ls["VersionId"] = args.key_version
+        k["KeyMarker"] = args.start_after
+        if args.version_id_marker:
+            k["VersionIdMarker"] = args.version_id_marker
+    return k
 
+
+def get_kwargs_acts(args):
+    k = {"Bucket": args.bucket}
     if args.action == "cp":
-        k_act["Bucket"] = args.dest_bucket
-        k_act["CopySource"] = {
+        k["Bucket"] = args.dest_bucket
+        k["CopySource"] = {
             "Bucket": args.bucket,
         }
-
-    return k_ls, k_act
+    return k
 
 
 def reverse_versions(objs):
@@ -236,47 +223,73 @@ def reverse_versions(objs):
 
 def run():
     n_tot = s_tot = 0
+    stop = False
+    act_on_key_data = {}
 
     args = get_args()
 
-    if args.key_version:
+    if args.key and (args.key_version or args.version_id_marker):
         args.versions = True
     if args.skip_current_version and not args.versions:
         return
 
     kwargs_s3_client_ls, kwargs_s3_client_action = get_kwargs_clients(args)
-    kwargs_s3_ls, kwargs_s3_action = get_kwargs_acts(args)
-
     s3_client_ls = boto3.client("s3", **kwargs_s3_client_ls)
     s3_client_action = boto3.client("s3", **kwargs_s3_client_action)
 
-    # For specific key/version
-    if args.key:
-        s3_client_get = boto3.client("s3", **kwargs_s3_client_ls)
-        resp = s3_client_get.get_object_attributes(**kwargs_s3_ls)
+    kwargs_s3_action = get_kwargs_acts(args)
 
-        print(
-            execute_s3_action(
-                args,
-                kwargs_s3_action,
-                s3_client_action,
-                {
-                    "key": args.key,
-                    "version": resp.get("VersionId"),
-                    "size": resp.get("ObjectSize", 0),
-                    "latest": False if args.versions else True,
-                    "date": resp.get("LastModified"),
-                    "n_tot": 1,
-                    "s_tot": resp.get("ObjectSize", 0),
-                },
+    # For specific key/version
+    if args.key and args.version_id_marker:
+        # If for a specific key want to start from a specific version using args.version_id_marker,
+        # simply use the key as marker for start_after
+        args.start_after = args.key
+    elif args.key:
+        kwargs_s3_get = {
+            "Bucket": args.bucket,
+            "Key": args.key,
+            "ObjectAttributes": ["ObjectSize"],
+        }
+        if args.key_version:
+            kwargs_s3_get["VersionId"] = args.key_version
+
+        s3_client_get = boto3.client("s3", **kwargs_s3_client_ls)
+        resp = s3_client_get.get_object_attributes(**kwargs_s3_get)
+
+        act_on_key_data = {
+            "key": args.key,
+            "version": resp.get("VersionId"),
+            "size": resp.get("ObjectSize", 0),
+            "latest": False if args.key_version else True,
+            "date": resp.get("LastModified"),
+        }
+
+        if args.key_version or not args.versions:
+            # Specific key with optionally specific version
+            act_on_key_data["n_tot"] = 1
+            act_on_key_data["s_tot"] = resp.get("ObjectSize", 0)
+            print(
+                execute_s3_action(
+                    args,
+                    kwargs_s3_action,
+                    s3_client_action,
+                    act_on_key_data,
+                )
             )
-        )
-        return
+            return
+
+        # For acting on all versions of a specific key with right order (current version at end)
+        args.start_after = args.key
+        args.version_id_marker = resp.get("VersionId")
+
+    kwargs_s3_ls = get_kwargs_ls(args)
 
     paginator = s3_client_ls.get_paginator("list_object_versions")
     response_iterator = paginator.paginate(**kwargs_s3_ls)
 
     for r in response_iterator:
+        if stop:
+            break
 
         with concurrent.futures.ThreadPoolExecutor(
             max_workers=args.max_s3_workers
@@ -291,6 +304,17 @@ def run():
                     list_objs = reverse_versions(list_objs)
 
             for p in list_objs:
+                if args.key and p.get("Key") != args.key:
+                    stop = True
+                    break
+
+                if args.skip_current_version and p.get("IsLatest"):
+                    # skip current
+                    continue
+                if not args.versions and not p.get("IsLatest"):
+                    # skip versions
+                    continue
+
                 n_tot += 1
                 s_tot += p.get("Size", 0)
                 s3_key_data = {
@@ -322,6 +346,19 @@ def run():
                     if s3_status:
                         print(s3_status)
 
+            if act_on_key_data and not args.skip_current_version:
+                # For acting on all versions of a specific key with right order (current version at end)
+                act_on_key_data["n_tot"] = n_tot + 1
+                act_on_key_data["s_tot"] = s_tot + act_on_key_data["size"]
+                print(
+                    execute_s3_action(
+                        args,
+                        kwargs_s3_action,
+                        s3_client_action,
+                        act_on_key_data,
+                    )
+                )
+
             if args.stop_on_error:
                 for future in future_to_stack:
                     future.cancel()
